#!/usr/bin/php
<?php
$options = getopt("T:hc:");
$type_option=array('stat','clean');
$type = 'stat';
if(isset($options["T"])&& !empty($options["T"]))
{
    $type = $options["T"];
    if(!in_array($type, $type_option))
    {
        echo "Bad type paramater\n";
        exit(1);
    }
}
$configuration_path = '/usr/local/syslog/public_html/';
if(isset($options["c"])&& !empty($options["c"]))
{
    $configuration_path = $options["c"];
}
require $configuration_path.'syslog.conf.php';
//connection à la base de donnée
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: '.$e->getMessage().'<br />';
	echo 'N° : '.$e->getCode();
    exit(1);
}
//
//Génération des statistiques
//
if($type == 'stat')
{
    //Statistique TOPHOST
    $queryupdate = $dbconnection->prepare("UPDATE syslog_stats SET stat_data=:data WHERE stat_name=:name;");
    $query = $dbconnection->prepare("SELECT fromhost, count(*) as nbr
                                    FROM systemevents
                                    WHERE receivedat > (NOW() - cast(:inter as interval))
                                    GROUP BY fromhost
                                    ORDER BY nbr DESC
                                    LIMIT 10;");
    statTopHost($query,$queryupdate,'1');
    statTopHost($query,$queryupdate,'6');
    statTopHost($query,$queryupdate,'12');
    statTopHost($query,$queryupdate,'24');

    //Statisque Par severité
    $query = $dbconnection->prepare("SELECT priority, count(*) as nbr
                                    FROM systemevents
                                    WHERE receivedat > (NOW() - cast(:inter as interval))
                                    GROUP BY priority
                                    ORDER BY nbr DESC");
    statSeverity($query,$queryupdate,'1');
    statSeverity($query,$queryupdate,'6');
    statSeverity($query,$queryupdate,'12');
    statSeverity($query,$queryupdate,'24');


    $query = $dbconnection->prepare("SELECT receivedat::date as date, count(*) as nbr
                                    FROM systemevents
                                    WHERE receivedat > (NOW()- interval '10 day')
                                    GROUP BY receivedat::date");
    $query->execute();
    $arr=array();
    while($row=$query->fetch(PDO::FETCH_OBJ))
    {
        $arr[]=array('date'=> $row->date,'nbr'=>$row->nbr);
    }
    $queryupdate->execute(array('name'=> 'LOG_BY_DAY','data'=>json_encode($arr)));
}
//
//Nettoyage DB
//
elseif($type =='clean')
{
    //chargement des règles
    $frule = fopen($configuration_path.RULES_FILE,'r+');
    if($frule===FALSE)
    {
        header('HTTP/1.1 500 Internal Server Error');
        echo "can't open rule file\n";
        exit(1);
    }
    $today = date("d/m/y H:i:s");
    $query = $dbconnection->prepare("DELETE FROM systemevents WHERE priority=:prior AND receivedat < (NOW() - cast( :inter as interval))");
    while (($buffer = fgets($frule, 4096)) !== false)
    {
        $value = explode('=',$buffer);
        $key = trim($value[0]);
        $day=trim($value[1]);
	if(!empty($day))
	{
        	$query->execute(array('prior'=>$priority_link[$key],':inter'=>"$day day"));
        	echo "$today # DELETE $key record that are older $day day. number delete: ".$query->rowCount()."\n";
	}
    }
    fclose($frule);
}

//
//FUNCTION
//
function statTopHost($selectquery,$updatequery,$interval)
{
    $selectquery->execute(array('inter'=> $interval.' hour'));
    $arr = array();
    while($row=$selectquery->fetch(PDO::FETCH_OBJ))
    {
        $arr[]=array('host'=> $row->fromhost,'nbr'=>$row->nbr);
    }
    $updatequery->execute(array('name'=> 'TOP_HOST_'.$interval.'HOUR','data'=>json_encode($arr)));
}
function statSeverity($selectquery,$updatequery,$interval)
{
    $selectquery->execute(array('inter'=> $interval.' hour'));
    global $priority_link;
    $priority_link_inv = array_flip($priority_link);

    $arr = array('emergency'=>0,'alert'=>0,'critical'=>0,'errors'=>0,'warning'=>0,'notice'=>0,'info'=>0,'debug'=>0);
    while($row=$selectquery->fetch(PDO::FETCH_OBJ))
    {
        $tmp = $priority_link_inv[$row->priority];
        $arr[$tmp]=$row->nbr;
    }
    $updatequery->execute(array('name'=> 'SEVERITY_'.$interval.'HOUR','data'=>json_encode($arr)));
}
?>
