-- Table: public.syslog_stats
CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;
-- DROP TABLE IF EXISTS public.syslog_stats;

CREATE TABLE IF NOT EXISTS public.syslog_stats
(
    stat_id serial,
    stat_name character varying(30) COLLATE pg_catalog."default" NOT NULL,
    stat_data json,
    CONSTRAINT syslog_stats_pkey PRIMARY KEY (stat_id),
    CONSTRAINT stats_name_unique UNIQUE (stat_name)
)
WITH (
    OIDS = FALSE
)
TABLESPACE pg_default;

ALTER TABLE IF EXISTS public.syslog_stats OWNER to rsyslog;
-- Table: public.syslog_rules

-- DROP TABLE IF EXISTS public.syslog_rules;

CREATE TABLE IF NOT EXISTS public.syslog_rules
(
    rule_id serial,
    rule_name character varying(50) COLLATE pg_catalog."default",
    rule_from_host character varying COLLATE pg_catalog."default",
    rule_message_contain character varying COLLATE pg_catalog."default",
    rule_match_number integer DEFAULT 1,
    rule_last_send timestamp without time zone,
    rule_priority smallint[],
    rule_match_time integer,
    rule_order integer DEFAULT 9999,
    CONSTRAINT syslog_rules_pkey PRIMARY KEY (rule_id)
)
WITH (
    OIDS = FALSE
)
TABLESPACE pg_default;

ALTER TABLE IF EXISTS public.syslog_rules
    OWNER to rsyslog;


CREATE INDEX IF NOT EXISTS systemsevents_host_index
    ON public.systemevents USING btree
    (fromhost COLLATE pg_catalog."default" ASC NULLS LAST)
    TABLESPACE pg_default;

CREATE INDEX systemsevents_severity_index
    ON public.systemevents USING btree
    (priority ASC NULLS LAST);


-- Trigger: notify_event

-- DROP TRIGGER IF EXISTS notify_event ON public.systemevents;

-- FUNCTION: public.trigger_notify_event()

-- DROP FUNCTION IF EXISTS public.trigger_notify_event();

CREATE OR REPLACE FUNCTION public.trigger_notify_event()
    RETURNS trigger
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE NOT LEAKPROOF
AS $BODY$
DECLARE
    v_sysrules RECORD;
	v_flag integer = 0;
	v_count integer = 0;
BEGIN
	RAISE NOTICE 'notice: %',NEW.fromhost;
	FOR v_sysrules IN
		SELECT rule_id,rule_name,rule_from_host,rule_priority,rule_message_contain,rule_match_number,rule_match_time
		FROM syslog_rules
		WHERE rule_from_host=NEW.fromhost OR rule_from_host IS NULL OR rule_from_host =''
		ORDER BY rule_order
	LOOP
		--S'il faut un match simple pour notifier
		IF v_sysrules.rule_match_number = 1
		THEN
			--test le niveau d'erreur s'il est présent ou pas
			IF NEW.priority = ANY (v_sysrules.rule_priority) OR v_sysrules.rule_priority IS NULL
			THEN
				IF v_sysrules.rule_message_contain IS NOT NULL
				THEN
					IF NEW.message LIKE '%'||v_sysrules.rule_message_contain||'%'
					THEN
						RAISE NOTICE 'SEND: PRIORITY & MESSAGE MATCH';
						v_flag=v_sysrules.rule_id;
						EXIT;
					END IF;
				ELSE
					RAISE NOTICE 'SEND: PRIORITY MATCH AND MESSAGE IS NULL';
					v_flag=v_sysrules.rule_id;
					EXIT;
				END IF;
			END IF;
		ELSE
			--s'il faut plus d'une erreur pour envoyer le message
			RAISE NOTICE 'RULE MULTIPLE: %',v_sysrules.rule_match_number;
			IF v_sysrules.rule_from_host IS NOT NULL AND v_sysrules.rule_from_host<>''
			THEN
				IF NEW.priority = ANY(v_sysrules.rule_priority)
				THEN
					IF v_sysrules.rule_message_contain IS NOT NULL
					THEN
						SELECT count(*) INTO v_count FROM systemevents
						WHERE fromhost = NEW.fromhost
							AND receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL)
							AND priority = ANY(v_sysrules.rule_priority)
							AND message LIKE '%'||v_sysrules.rule_message_contain||'%';
					ELSE
						SELECT count(*) INTO v_count FROM systemevents
						WHERE fromhost = NEW.fromhost
							AND receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL)
							AND priority = ANY(v_sysrules.rule_priority);
					END IF;
				ELSIF v_sysrules.rule_priority IS NULL
				THEN
					SELECT count(*) INTO v_count FROM systemevents
					WHERE fromhost = NEW.fromhost
						AND receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL);
				END IF;
			ELSE
				RAISE NOTICE 'RULE GENERAL: %',v_sysrules.rule_name;
				IF NEW.priority = ANY(v_sysrules.rule_priority)
				THEN
					IF v_sysrules.rule_message_contain IS NOT NULL
					THEN
						SELECT count(*) INTO v_count FROM systemevents
						WHERE receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL)
							AND priority = ANY(v_sysrules.rule_priority)
							AND message LIKE '%'||v_sysrules.rule_message_contain||'%';
					ELSE
						SELECT count(*) INTO v_count FROM systemevents
						WHERE receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL)
							AND priority = ANY(v_sysrules.rule_priority);
					END IF;
				ELSIF v_sysrules.rule_priority IS NULL
				THEN
					RAISE NOTICE 'RULE priority null:';
					IF v_sysrules.rule_message_contain IS NOT NULL
					THEN
						RAISE NOTICE 'RULE message : %', v_sysrules.rule_message_contain;
						SELECT count(*) INTO v_count FROM systemevents
						WHERE receivedat > (NOW() - (v_sysrules.rule_match_time||' seconds')::INTERVAL)
							AND message LIKE '%'||v_sysrules.rule_message_contain||'%';
					END IF;
				END IF;
				RAISE NOTICE 'RULE GENERAL: % > % in % ',v_count,v_sysrules.rule_match_number, v_sysrules.rule_match_time;
			END IF;
			IF v_count > v_sysrules.rule_match_number
			THEN
				v_flag=v_sysrules.rule_id;
				EXIT;
			END IF;
		END IF;
	END LOOP;
	IF v_flag > 0
	THEN
		RAISE NOTICE 'SEND NOTIFY: %',NEW.fromhost;
		PERFORM pg_notify('syslog', row_to_json(NEW)::TEXT);
		UPDATE syslog_rules SET rule_last_send=NOW() WHERE rule_id=v_flag;
	END IF;
	RETURN NEW;
END;
$BODY$;

ALTER FUNCTION public.trigger_notify_event()
    OWNER TO rsyslog;


CREATE TRIGGER notify_event
    AFTER INSERT
    ON public.systemevents
    FOR EACH ROW
    EXECUTE PROCEDURE public.trigger_notify_event();


INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'TOP_HOST_1HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'TOP_HOST_6HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'TOP_HOST_12HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'TOP_HOST_24HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'SEVERITY_1HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'SEVERITY_6HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'SEVERITY_12HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'SEVERITY_24HOUR');
INSERT INTO public.syslog_stats(stat_id, stat_name)VALUES (DEFAULT,'LOG_BY_DAY');
